##############################################################################
#
# Copyright (c) 2007 EVI All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from osv import osv, fields
from tools.sql import drop_view_if_exists


class account_mistaken_journal_period(osv.osv):

    _name = "account.mistaken_journal_period"

    _description = "Errors on account.move.line in database"

    _auto = False

    _columns = {
        'line_id': fields.many2one('account.move.line', 'Entry line'),
        'move_name': fields.char('Move Number', size=64, readonly=True),
        'move_period_id': fields.many2one('account.period', 'Move Period', readonly=True),
        'move_date': fields.date('Move Date', readonly=True),
        'move_state': fields.selection([('draft','Draft'), ('posted','Posted')], 'Move Status', readonly=True),
        'move_journal_id': fields.many2one('account.journal', 'Move Journal', readonly=True),
        'line_name': fields.char('Line Number', size=64, readonly=True),
        'line_period_id': fields.many2one('account.period', 'Line Period', readonly=True),
        'line_date': fields.date('Line Date', readonly=True),
        'line_state': fields.selection([('draft','Draft'), ('valid','Valid')], 'Line Status', readonly=True),
        'line_journal_id': fields.many2one('account.journal', 'Line Journal', readonly=True),
        'debit': fields.float('Line Debit', digits=(16,2), readonly=True),
        'credit': fields.float('Line Credit', digits=(16,2), readonly=True),
    }


    def init(self, cr):
        drop_view_if_exists(cr,'account_mistaken_journal_period')
        cr.execute("""
            CREATE VIEW account_mistaken_journal_period AS
            SELECT account_move_line.id AS id, 
                account_move_line.id AS line_id, 
                account_move.name AS move_name, account_move.period_id AS move_period_id, account_move.date AS move_date, 
                    account_move.state AS move_state, account_move.journal_id AS move_journal_id, 
                account_move_line.name AS line_name, account_move_line.period_id AS line_period_id, account_move_line.date AS line_date, 
                    account_move_line.state AS line_state, account_move_line.journal_id AS line_journal_id, 
                account_move_line.debit, account_move_line.credit
            FROM account_move
                JOIN account_move_line ON account_move_line.move_id = account_move.id
            WHERE account_move_line.journal_id != account_move.journal_id
                OR account_move_line.period_id != account_move.period_id
            ORDER BY account_move_line.id        
        """)

account_mistaken_journal_period()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
