# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2007 EVI All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
from osv import osv, fields
from tools.sql import drop_view_if_exists
import release


class fiscalyear_close_erroraccount(osv.osv):
    _name = "fiscalyear.close.erroraccount"
    _description = "Account Error"
    _auto = False
    _order = "id"
    _columns = {
        'account_id': fields.many2one('account.account', 'Account', required=True, ondelete="cascade"),
    }
    def init(self, cr):
        drop_view_if_exists(cr, 'fiscalyear_close_error_accountlines')
        drop_view_if_exists(cr, 'fiscalyear_close_errorlines')
        drop_view_if_exists(cr, 'fiscalyear_close_erroraccount')
        cr.execute("""create or replace view fiscalyear_close_erroraccount as (
            SELECT ac.id,ac.id as account_id from account_account ac where id in (select parent_id from account_account group by parent_id) and type<>'view' )""")

fiscalyear_close_erroraccount()


class fiscalyear_close_errorlines(osv.osv):
    _name = "fiscalyear.close.errorlines"
    _description = "Account Lines"
    _auto = False
    _order = "id"
    _columns = {
        'date': fields.date('Effective date', required=True),
        'name': fields.char('Nom',size=256,readonly=True),
        'account_id': fields.many2one('account.account', 'Account', required=True, ondelete="cascade"),
        'move_id': fields.many2one('account.move', 'Move', ondelete="cascade", states={'valid':[('readonly',True)]}, help="The move of this entry line.", select=2),
        'debit': fields.float('Debit', digits=(16,2)),
        'credit': fields.float('Credit', digits=(16,2)),
        'ref': fields.char('Ref.', size=32),
        'reconcile_id': fields.many2one('account.move.reconcile', 'Reconcile', readonly=True, ondelete='set null', select=2),
        'amount_currency': fields.float('Amount Currency', help="The amount expressed in an optionnal other currency if it is a multi-currency entry."),
        'currency_id': fields.many2one('res.currency', 'Currency', help="The optionnal other currency if it is a multi-currency entry."),
        'period_id': fields.many2one('account.period', 'Period', required=True),
        'journal_id': fields.many2one('account.journal', 'Journal', required=True),
        'partner_id': fields.many2one('res.partner', 'Partner Ref.'),
        'state': fields.selection([('draft','Draft'), ('valid','Valid')], 'State', readonly=True),
    }
    def init(self, cr):
        drop_view_if_exists(cr,'fiscalyear_close_errorlines')
        cr.execute("""create or replace view fiscalyear_close_errorlines as (
            SELECT id,date,name,account_id,move_id,debit,credit,ref,reconcile_id,amount_currency,currency_id, period_id,journal_id,partner_id,state  
            FROM account_move_line where account_id in (select id from fiscalyear_close_erroraccount))""")
fiscalyear_close_errorlines()

class fiscalyear_close_error_accountlines(osv.osv):
    """ dénombrement des lgines d'écriture comptable, dont le compte est mal paramétré.
            C-a-d, sur des compte de type normaux (non vue) qui sont des parent (qui ont des enfants)
    """
    _name = "fiscalyear.close.error.accountlines"
    _description = "Erronous account number with lines"
    _auto = False

    _columns = {
        'account_id': fields.many2one('account.account', 'Account', readonly=True),
        'parent_id': fields.many2one('account.account', 'Parent', readonly=True),
        'period_id': fields.many2one('account.period', 'Move Period', readonly=True),
        'number': fields.integer('Number of lines', readonly=True),
    }

    def init(self, cr):
        drop_view_if_exists(cr, 'fiscalyear_close_error_accountlines')
        cr.execute("""CREATE OR REPLACE VIEW fiscalyear_close_error_accountlines AS (
                            SELECT MIN(account_move_line.id) AS id, account_id, parent_id, period_id, COUNT(DISTINCT(account_move_line.id)) AS number
                            FROM account_move_line, account_account
                                WHERE account_id in (SELECT id FROM fiscalyear_close_erroraccount)
                                AND account_move_line.account_id = account_account.id
                            GROUP BY account_id, parent_id, period_id
                        )""")

fiscalyear_close_error_accountlines()

class fiscalyear_close_error_lines_view(osv.osv):
    _name = "fiscalyear.close.error_lines_view"
    _description = "Account Lines"
    _auto = False
    _order = "id"
    _columns = {
        'date': fields.date('Effective date', required=True),
        'name': fields.char('Nom',size=256,readonly=True),
        'account_id': fields.many2one('account.account', 'Account', required=True, ondelete="cascade"),
        'move_id': fields.many2one('account.move', 'Move', ondelete="cascade", states={'valid':[('readonly',True)]}, help="The move of this entry line.", select=2),
        'debit': fields.float('Debit', digits=(16,2)),
        'credit': fields.float('Credit', digits=(16,2)),
        'ref': fields.char('Ref.', size=32),
        'reconcile_id': fields.many2one('account.move.reconcile', 'Reconcile', readonly=True, ondelete='set null', select=2),
        'amount_currency': fields.float('Amount Currency', help="The amount expressed in an optionnal other currency if it is a multi-currency entry."),
        'currency_id': fields.many2one('res.currency', 'Currency', help="The optionnal other currency if it is a multi-currency entry."),
        'period_id': fields.many2one('account.period', 'Period', required=True),
        'journal_id': fields.many2one('account.journal', 'Journal', required=True),
        'partner_id': fields.many2one('res.partner', 'Partner Ref.'),
        'state': fields.selection([('draft','Draft'), ('valid','Valid')], 'State', readonly=True),
    }
    def init(self, cr):
        drop_view_if_exists(cr,'fiscalyear_close_error_lines_view')
        cr.execute("""create or replace view fiscalyear_close_error_lines_view AS (
                        SELECT account_move_line.id, account_move_line.date, account_move_line.name, account_id, move_id, debit, credit, ref, reconcile_id, 
                                amount_currency, account_move_line.currency_id, period_id, journal_id, partner_id, state 
                        FROM account_move_line JOIN account_account ON account_move_line.account_id = account_account.id 
                        WHERE account_account.user_type IN (SELECT id FROM account_account_type WHERE lower(name) = 'view') OR account_account.type = 'view'
                    )""")

fiscalyear_close_error_lines_view()


class fiscalyear_close_erroraccounttype(osv.osv):
    _name = "fiscalyear.close.erroraccounttype"
    _description = "Account Type"
    _auto = False
    _order = "code"
    _columns = {
        'name': fields.char('Name', size=128, required=True, select=True),
        'code': fields.char('Code', size=64, required=True),
        'user_type': fields.many2one('account.account.type', 'Account Type', required=True),
        #'type_name': fields.char('Acc. Type Name', size=64, required=True, translate=True),
        'close_method': fields.selection([('none', 'None'), ('balance', 'Balance'), ('detail', 'Detail'), ('unreconciled', 'Unreconciled')], 'Deferral Method', required=True),
    }
    def init(self, cr):
        drop_view_if_exists(cr,'fiscalyear_close_erroraccounttype')
        cr.execute("""
            CREATE OR REPLACE VIEW fiscalyear_close_erroraccounttype AS (
                SELECT aa.id, aa.code AS code, aa.name AS name, aa.user_type AS user_type, aat.close_method AS close_method
                FROM account_account aa, account_account_type aat
                WHERE aa.user_type=aat.id
            )""")
fiscalyear_close_erroraccounttype()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
