#
#  account_move_line.py
#  c2c_fiscal_year_close
#
#  Created by Nicolas Bessi on 28.01.2008.
#  Copyright (c) 2008 CamptoCamp. All rights reserved.
#

import netsvc
logger = netsvc.Logger()
from osv import osv, fields
#import pooler
import time
import netsvc
from tools.translate import _
from datetime import datetime as date

#from sql import drop_view_if_exists

class account_move_line(osv.osv):
    """Class that inherits account_move_line that add a new reconcile
        method that is less restrictive """

    _inherit = "account.move.line"
    _columns = {

        ## link to the origini unreconciles entries
        "origin_link" : fields.many2one('account.move.line',
            'Parent line', select=True),
    }

    ## @param self The object pointer.
    ## @param cr a psycopg cursor
    ## @param uid res.user.id that is currently loged
    ## @param type auto or manual (auto is used for wizard)s
    ## @parma writeoff_acc_id id of write off account
    ## @parma writeoff_period_id id of write off period
    ## @parma writeoff_journal_id id of write off journal
    ## @parma context a context dict
    ## @return id of the created reconciliation or False
    def easy_reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False,
         writeoff_period_id=False, writeoff_journal_id=False, context={}):
        """ Reconciliation Method that return a complete reconciliation id"""
#        print "DEBUG: easy_reconcile(%r ; %r ; %r ; %r ; %r ; %r)" % (ids, type, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context)
        id_set = ','.join(map(str, ids))
        lines = self.read(cr, uid, ids, context=context)
        unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
        credit = debit = 0.0
        currency = 0.0
        account_id = False
        partner_id = False
        for line in unrec_lines:
            if line['state'] <> 'valid':
                raise osv.except_osv('Error',
                        'Entry "%s" is not valid !' % line.name)
            credit += line['credit']
            debit += line['debit']
            currency += line['amount_currency'] or 0.0
            account_id = line['account_id'][0]
            partner_id = (line['partner_id'] and line['partner_id'][0]) or False
        writeoff = debit - credit

        # Ifdate_p in context => take this date
        if context.has_key('date_p') and context['date_p']:
            date=context['date_p']
        else:
            date = time.strftime('%Y-%m-%d')

        cr.execute('SELECT account_id, reconcile_id \
                FROM account_move_line \
                WHERE id IN ('+id_set+') \
                GROUP BY account_id,reconcile_id')
        r = cr.fetchall()
        account = self.pool.get('account.account').browse(cr, uid, account_id, context=context)
        if (not self.pool.get('res.currency').is_zero(cr, uid, account.company_id.currency_id, writeoff)) or \
           (account.currency_id and (not self.pool.get('res.currency').is_zero(cr, uid, account.currency_id, currency))):
            if not writeoff_acc_id:
                raise osv.except_osv('Warning',
                    'You have to provide an account for the write off entry !')
            if writeoff > 0:
                debit = writeoff
                credit = 0.0
                self_credit = writeoff
                self_debit = 0.0
            else:
                debit = 0.0
                credit = -writeoff
                self_credit = 0.0
                self_debit = -writeoff
            # If comment exist in context, take it
            if context.has_key('comment') and context['comment']:
                libelle=context['comment']
            else:
                libelle='Write-Off'

            writeoff_lines = [
                (0, 0, {
                    'name':libelle,
                    'debit':self_debit,
                    'credit':self_credit,
                    'account_id':account_id,
                    'date':date,
                    'partner_id':partner_id,
                    'currency_id': account.currency_id.id or False,
                    'amount_currency': account.currency_id.id and -currency or 0.0
                }),
                (0, 0, {
                    'name':libelle,
                    'debit':debit,
                    'credit':credit,
                    'account_id':writeoff_acc_id,
                    'date':date,
                    'partner_id':partner_id
                })
            ]

            name = 'Write-Off'
            if writeoff_journal_id:
                journal = self.pool.get('account.journal').browse(
                    cr, uid, writeoff_journal_id)
                if journal.sequence_id:
                    name = self.pool.get('ir.sequence').get_id(
                        cr, uid, journal.sequence_id.id)

            writeoff_move_id = self.pool.get('account.move').create(
                cr,
                uid,
                {
                    'name': name,
                    'period_id': writeoff_period_id,
                    'journal_id': writeoff_journal_id,
                    'state': 'draft',
                    'line_id': writeoff_lines
                    }
            )

            writeoff_line_ids = self.search(cr, uid, [('move_id', '=', writeoff_move_id),
                ('account_id', '=', account_id)])
            ids += writeoff_line_ids

        r_id = self.pool.get('account.move.reconcile').create(
            cr,
            uid, {
                'type': type,
                'line_id': map(lambda x: (4,x,False), ids)
        })
        # the id of the move.reconcile is written in the move.line (self) by the create method above
        # because of the way the line_id are defined: (4, x, False)
        wf_service = netsvc.LocalService("workflow")
        for id in ids:
            wf_service.trg_trigger(uid, 'account.move.line', id, cr)
        return r_id


    def reconcile_two_years(self, cr, uid, move_id, context=None):
        """ Unreaconcile line that have been done upon two fiscal years.
                Redo reconciliation, between opening (new year) lines and passed year lines.
                For example : invoice was done on closed fiscal year. Then payment is done on (new) open fiscal year.
                At this point, when closing fiscal year, we need to break reconciliation between 2 year, to redo it twice
                    (one for each fiscal year : invoice with closing move, and opening move with payment)
            @params move_id (id) : ID of move opening move
        """
        if context is None:
            context = {}
        reconcil_obj = self.pool.get('account.move.reconcile')
        new_rec_ids = []
        old_rec_ids = {}
        line_ids = self.search(cr, uid, [('move_id', '=', move_id)], context=context)
        for new_line in self.browse(cr, uid, line_ids, context=context):
        # Find passed year line that is in origin of actual/new one AND check if it's reconciled
            if new_line.origin_link.reconcile_id:
                reconcile_id = new_line.origin_link.reconcile_id.id
                if reconcile_id not in old_rec_ids:      # Check if it has not already been done (as reconcile is for several lines)
                    old_rec_ids[reconcile_id] = []
                    for rec_line_id in new_line.origin_link.reconcile_id.line_id:
                        old_rec_ids[reconcile_id].append(rec_line_id.id)  # Reconcile new fiscal year lines

                old_rec_ids[reconcile_id].remove(new_line.origin_link.id)           # remove old line
                old_rec_ids[reconcile_id].append(new_line.id)                       # add new line
                if date.strptime(new_line.origin_link.date, "%Y-%m-%d").year + 1 != date.strptime(new_line.date, "%Y-%m-%d").year:
                    logger.notifyChannel('account.move.line', netsvc.LOG_WARNING, 'reconcile : %s/%d' % (new_line.origin_link.reconcile_id.name, new_line.origin_link.reconcile_id.id ))
                    logger.notifyChannel('account.move.line', netsvc.LOG_WARNING, 'old : %s | date : %s' % (new_line.origin_link.name, new_line.origin_link.date))
                    logger.notifyChannel('account.move.line', netsvc.LOG_WARNING, 'new : %s | date : %s' % (new_line.name, new_line.date))
                    raise osv.except_osv( 'ERROR', 'ERROR DATE')

        # Reconcile (invoice on closed year with payment on open year)  / No writte-Off
        for reconcil_id in old_rec_ids.keys():
            reconcil = reconcil_obj.browse(cr, uid, reconcil_id, context=context)
            logger.notifyChannel('account.move.line', netsvc.LOG_DEBUG, 'reconcile : %s / %d' % (reconcil.name, reconcil.id))
            logger.notifyChannel('account.move.line', netsvc.LOG_DEBUG, 'len(line) : %s' % repr(len(old_rec_ids[reconcil_id])))
            for l in self.browse(cr, uid, old_rec_ids[reconcil_id], context=context):
                logger.notifyChannel('account.move.line', netsvc.LOG_DEBUG, 'ids : (%s/%d) | credit = %d | debit = %d | date = %s | compte = %s' % (l.name, l.id, l.credit, l.debit, l.date, l.account_id.code ))
            reconcil_obj.unlink(cr, uid, reconcil_id, context=context)
            new_rec_ids.append(self.reconcile(cr, uid, old_rec_ids[reconcil_id], context=context))

        return new_rec_ids

account_move_line()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
