# -*- encoding: utf-8 -*-
#
#  fiscal_year_close.py
# 
#
#  Created by Nicolas Bessi on 28.01.09.
#  Copyright (c) 2009 CamptoCamp. All rights reserved.
#

import wizard
import time
import osv
import pooler

##TODO Replace journal base booking by journal.period 
_transaction_form = '''\
<?xml version="1.0"?>
<form string="Cloture">
    <separator string="Fiscal Years" colspan="4"/>
    <field name="fy_id" colspan="3" width="250"/>
    <newline/>
    <field name="fy2_id" colspan="3"/>
    <newline/>
    <field name="company_id" colspan="3"/>
    <field name="report_new"/>
    <field name="report_name" colspan="3"/>
    <separator string="If profit " colspan="4"/>
    <field name="bilan_account_prof" colspan="3"/>
    <field name="pl_prof" colspan="3"/>
    <separator string="If loss" colspan="4"/>
    <field name="bilan_account_loss" colspan="3"/>
    <field name="pl_loss" colspan="3"/>
    <separator string="Confirmation and warning" colspan="4"/>
<field name="sure"/>
</form>'''

_transaction_fields = {
    ## If this box is activated the wizard will try to separate the object by they company_id attribute
    
    ## fiscal year to close
    'fy_id': {'string':'Fiscal Year to close', 'type':'many2one', 
        'relation': 'account.fiscalyear','required':True, 'domain':[('state','=','draft')]},
        
    ## fiscal year to open
    'fy2_id': {'string':'New Fiscal Year', 'type':'many2one', 
        'relation': 'account.fiscalyear', 'domain':[('state','=','draft')], 'required':True},
        
    ## if this is done  the new Balance writing will be created
    'report_new': {'string':'Create new entries', 'type':'boolean', 
        'required':True, 'default': lambda *a:True},
        
    ## wording of new entries
    'report_name': {'string':'Wording of new entries', 'type':'char',
        'size': 64, 'required':True},
    
    ## bilan account in case of profit 
    'bilan_account_prof': {'string':'Bilan account for profit', 'type':'many2one',
        'relation': 'account.account', 'required':True},
    
    ## Profit and loss in case of profit 
    'pl_prof': {'string':'Profit and loss account for profit', 'type':'many2one', 
        'relation': 'account.account', 'required':True},
        
    ## bilan account in case of loss 
    'bilan_account_loss': {'string':'Bilan account for loss', 'type':'many2one', 
        'relation': 'account.account', 'required':True},
        
    ## Profit and loss in case of loss 
    'pl_loss': {'string':'Profit and loss account for profit', 'type':'many2one', 
        'relation': 'account.account', 'required':True},
        
    ## Profit and loss in case of loss 
    'company_id': {'string':'Company', 'type':'many2one', 
        'relation': 'res.company', 'required':True},
        
    ## confirm the fiscalyear close
    'sure': {'string':'Check this box to confirm \
!!!Be sure that your base has a backup!!!', 
        'type':'boolean'},
}


## @param x first date string to compare
## @param y seconde date string to compare
## @return 1, -1 or 0 please see http://wiki.python.org/moin/HowTo/Sorting for more infos 
def date_compare(x,y):
    """ Classic comparator function use by comp"""
    if time.strptime(x.date_start,'%Y-%m-%d') >\
    time.strptime(y.date_start,'%Y-%m-%d'):
        return 1
    elif time.strptime(x.date_start,'%Y-%m-%d') ==\
    time.strptime(y.date_start,'%Y-%m-%d'):
        return 0
    elif time.strptime(x.date_start,'%Y-%m-%d') <\
    time.strptime(y.date_start,'%Y-%m-%d'):
        return -1

## @param cr a psycopg cursor
## @param uid res.user.id that is currently loged
## @param result a psycopg result set return by cr.fetchall() that contain a list of account move line
## @param fy2 fiscal year to close
## @parma pool a OpenERP database pooler
## @return a filtered result set see doc string for infos 	
def _filter_result_to_reconcile(cr, uid, result, fy1, start_period, pool):
    """ Return a list of account.move.line to be reported on the next fiscal year.
    This filter is used only for account with unreconciled closing method. This will retrieve
    all the account move line to be reported to the new year. The should be unreconciled or 
    have a reconciliation line with a date that is bigger than the next fiscal year start date"""

    ### this is the tricky moment 
    to_return = []
    #if there are no result we have done the turn of the offset we return None in order to break the while clause
    if not result :
        return None
    ## the date stop is the end date of the fiscal year to close
    date_stop = time.strptime(fy1.date_stop, '%Y-%m-%d')
    for line in result :
        acc_move_line = pool.get('account.move.line').browse(cr, uid, line['id'])
                                                        
        
        ## if the account has an origin that mean a move line that was not reconciled on the precedent year
        ## if it is the case we check if the writing was reconciled during this year 
        if acc_move_line.origin_link :
            origin_line =  acc_move_line.origin_link
            ## we have the origin account move line and
            # we retriev the corresponding reconciliations
            if origin_line.reconcile_id :
                rec_lines = origin_line.reconcile_id.line_id
                for rec_line in rec_lines :
                    ## if the line was reconciled next year 
                    if time.strptime(rec_line.period_id.date_start, '%Y-%m-%d')\
                    >= date_stop :
                        del(line['id'])
                        ## we keep the track of this it may slower the next 
                        ## fiscal year close but if they unreconcile the line 
                        ## next year we will handle the case.
                        to_return.append(line)
                        break
            else :
                ## the move has to be reported
                del(line['id'])
                to_return.append(line)
                continue 
        ###---------------------- stop handling  the reconciliation origin ------------------------
        
        if acc_move_line.reconcile_id :
            rec_lines = acc_move_line.reconcile_id.line_id
            if rec_lines :
                for rec_line in rec_lines :
                    if time.strptime(rec_line.period_id.date_start, '%Y-%m-%d')\
                    >= date_stop :
                        line['origin_link'] = line['id']
                        del(line['id'])
                        to_return.append(line)
                        break
        else :
            line['origin_link'] = line['id']
            del(line['id'])
            to_return.append(line)
    return to_return

## @param cr a psycopg cursor
## @param uid res.user.id that is currently loged
## @param fy2 fiscal year to open
## @parma start_period the browse record of the period to open 
## @parma m_id the id of the move to reconcile
## @parma pool a OpenERP database pooler
def reconcile_new_lines(cr, uid, fy2, start_period, m_id, pool) :
    """Reconcile all the line of the new fiscal year opening journal"""
#    print "DEBUG: reconcile_new_lines(%r, %r, %r)" % (fy2, start_period, m_id)
    account_m_line_obj = pool.get('account.move.line')
    move_ids = account_m_line_obj.search(cr, uid, [('move_id', '=', m_id)])

#    print "DEBUG: easy_reconcile(%r, %r, %r)" % (len(move_ids), start_period.id, fy2.start_journal_period_id.id)
    reconcil_id = account_m_line_obj.easy_reconcile(
        cr,
        uid, 
        move_ids, 
        writeoff_period_id=start_period.id,
        writeoff_journal_id=fy2.start_journal_period_id.id,
    )
#    print "DEBUG: reconcile_new_lines > %r" % reconcil_id


## @param cr a psycopg cursor
## @param uid res.user.id that is currently loged
## @param data the dict that contain the form info and the ids to be treated
## @param fy browse record of the fiscal year to close
## @param fy browse record of the fiscal year to open
## @parma pool a OpenERP database pooler
##/ @param m_id open move_id
def _profLoss_to_bilan(cr, uid, data, fy, fy2, pool,m_id) :
    """ compute the closing writing and save it on the close journal of the fiscal year"""
    
    bilan_account_loss = data['form']['bilan_account_loss']
    bilan_account_prof = data['form']['bilan_account_prof']
    pl_loss = data['form']['pl_loss']
    pl_prof = data['form']['pl_prof']
    
    cr.execute('select id from account_account WHERE active and company_id = %s',(data['form']['company_id'],))
    
    res = cr.fetchall()
    ids = map(lambda x: x[0],res)

    summ = pool.get('account.fiscalyear').compute_profit_loss(cr, uid, fy.id)

    ## get last period of fiscal year
    periods_ids = pool.get('account.period').search(cr,uid,[('fiscalyear_id','=',fy.id),('special','=',True)]) 
    periods = pool.get('account.period').browse(cr,uid,periods_ids) 

    periods.sort(date_compare)
    period = periods[len(periods)-1]
    ##we generate a move
   
    #name = pool.get('ir.sequence').get_id(cr, uid,fy.end_journal_period_id.sequence_id.id)
    #move_id = pool.get('account.move').create(cr, uid, {
    #        'name': name,
    #       'ref':'ecritures de soldes des comptes de bilan',
    #        'period_id': period.id,
    #        'journal_id': 5, # fy.end_journal_period_id.id,
    #   })
    #lines_open_ids = pool.get('account.move.line').search(cr, uid,[('move_id','=',m_id)])
    #for line_id in lines_open_ids:
    #    line=pool.get('account.move.line').browse(cr, uid,line_id)
    #    line_id1 = pool.get('account.move.line').create(
    #        cr, 
    #        uid, 
    #        {
    #            'name': line.name.replace(data['form']['report_name'],"Close"),
    #            'account_id': line.account_id.id,
    #            'journal_id': 5,#fy.end_journal_period_id.id,
    #            'move_id' : move_id,
    #            'period_id':period.id,
    #            'date': fy.date_stop,
    #            'debit': line.credit,
    #            'credit': line.debit,
    #        }, 
    #        {
    #            'journal_id': fy.end_journal_period_id.id,
    #            'period_id': period.id
    #        }
    #    )	
    name = pool.get('ir.sequence').get_id(cr, uid,fy.end_journal_period_id.sequence_id.id)

    move_id = pool.get('account.move').create(cr, uid, {
            'name': name,
            'period_id': period.id,
            'journal_id':  fy.end_journal_period_id.id,
        })   
    ##if it is a profit 
    if summ < 0.0 :
        
        line_id1 = pool.get('account.move.line').create(
            cr, 
            uid, 
            {
                'name': 'Profit of the year',
                'account_id': bilan_account_prof,
                'journal_id': fy.end_journal_period_id.id,
                'move_id' : move_id,
                'period_id':period.id,
                'date': fy.date_stop,
                'debit': 0.0,
                'credit': abs(summ),
            }, 
            {
                'journal_id': fy.end_journal_period_id.id,
                'period_id': period.id
            }
        )	
        
        line_id2 = pool.get('account.move.line').create(
            cr, 
            uid, 
            {
                'name': 'Profit of the year',
                'account_id': pl_prof,
                'journal_id': fy.end_journal_period_id.id,
                'move_id' : move_id,
                'period_id':period.id,
                'date': fy.date_stop,
                'debit':abs(summ), 
                'credit': 0.0 ,
            }, 
            {
                'journal_id': fy.end_journal_period_id.id,
                'period_id': period.id
            }
        )
    elif  summ > 0.0 :
    
        line_id1 = pool.get('account.move.line').create(
            cr, 
            uid, 
            {
                'name': 'Loss of the year',
                'account_id': bilan_account_loss,
                'journal_id': fy.end_journal_period_id.id,
                'move_id' : move_id,
                'period_id':period.id,
                'date': fy.date_stop,
                'debit':  abs(summ),
                'credit': 0.0,
            }, 
            {
                'journal_id': fy.end_journal_period_id.id,
                'period_id': period.id
            }
        )	
        
        line_id2 = pool.get('account.move.line').create(
            cr, 
            uid, 
            {
                'name': 'Loss of the year',
                'account_id': pl_loss,
                'journal_id': fy.end_journal_period_id.id,
                'move_id' : move_id,
                'period_id':period.id,
                'date': fy.date_stop,
                'debit': 0.0,
                'credit':  abs(summ),
            }, 
            {
                'journal_id': fy.end_journal_period_id.id,
                'period_id': period.id
            }
        )


## @param cr a psycopg cursor
## @param uid res.user.id that is currently loged
## @param data the dict that contain the form info and the ids to be treated
## @parma pool a OpenERP database pooler
## @parma context a context dict
## @return True if all input are valid else raise a wizard error
def _test_input(cr, uid, data, pool, context):
    """Check if the fiscal year and the account and the wizard form inputs are valids"""
    ##validate the fy
    fy = pool.get('account.fiscalyear').browse(cr, uid, data['form']['fy_id'])
    fy2 = pool.get('account.fiscalyear').browse(cr, uid, data['form']['fy2_id'])
    ## date of cloture
    date_stopfy = time.strptime(fy.date_stop, '%Y-%m-%d')
    ## date op openning
    date_startfy2 = time.strptime(fy2.date_start, '%Y-%m-%d')
    
    if (fy.state == 'done') or (fy2.state == 'done') :
        raise wizard.except_wizard('UserError',
                'One of the selected fiscal year is aleready closed') 
                
    if date_stopfy > date_startfy2 :
        raise wizard.except_wizard('UserError',
                'The Fiscal Year must be successive') 
    ## we get all the fiscal year start date that are not in the selecte one
   
    cr.execute('select name, date_start, state from account_fiscalyear where id not in(%s,%s)',(fy.id, fy2.id))
    res = cr.dictfetchall()
    if res :
    ## we test if ther is a fiscal year open between 
        for fy_to_test in res :
            date_to_test = time.strptime(fy_to_test['date_start'], '%Y-%m-%d')
            if date_to_test > date_stopfy \
            and date_to_test < date_startfy2 :
                raise wizard.except_wizard('UserError',
                'there is a fiscal year between the one you want to close : '\
                +fy_to_test['name']) 
    ## we test if all the previous fiscal year are closed
        for fy_to_test in res :
            date_to_test = time.strptime(fy_to_test['date_start'], '%Y-%m-%d')
            if date_to_test < date_stopfy \
            and fy_to_test['state'] != 'done' :
                raise wizard.except_wizard('UserError',
                'there can not be an unclose fiscal year before the one you want to close : '\
                +fy_to_test['name']) 
    user_ci = pool.get('res.users').browse(cr, uid, uid, context=context).company_id.id
    if user_ci != data['form']['company_id'] :
        raise wizard.except_wizard('UserError',
                'You can not close the fiscal year of an other\
                 company you have to log into the other company') 
    return True  

def _data_load(self, cr, uid, data, context):
    data['form']['report_name'] = 'End of Fiscal Year Entry'
    return data['form']

## @param Self Pointer to the object
## @param cr a psycopg cursor
## @param uid res.user.id that is currently loged
## @param data the dict that contain the form info and the ids to be treated
## @parma context a context dict
## @return an empty dict 
def _data_save(self, cr, uid, data, context):
    """Do the closing see the terp file or the tech spec for details"""
    pool = pooler.get_pool(cr.dbname)
    _test_input(cr, uid, data, pool, context)
    
    if not data['form']['sure']:
        raise wizard.except_wizard('UserError', 
            'Closing of fiscal year canceled, please check the box !')
    

    fy_id = data['form']['fy_id']
    fy2_id = data['form']['fy2_id']
    fy = pool.get('account.fiscalyear').browse(cr, uid, fy_id)
    fy2 = pool.get('account.fiscalyear').browse(cr, uid, fy2_id)
    
    periods_ids = pool.get('account.period').search(cr,uid,[('fiscalyear_id','=',fy2.id),('special','=',True)]) 
    
    periods = pool.get('account.period').browse(cr,uid,periods_ids) 
    periods.sort(date_compare)
    period = periods[0]
    journal = fy2.start_journal_period_id
    
    #if not journal.default_credit_account_id or not journal.default_debit_account_id:
    #    raise wizard.except_wizard('UserError','The journal must have default credit and debit account')
    #if not journal.centralisation:
    #    raise wizard.except_wizard('UserError', 'The journal must have centralised counterpart')
    #
    query_line = pool.get('account.move.line')._query_get(cr, uid,obj='account_move_line', context={'fiscalyear': fy_id})	
    print "Query line ",query_line        
    ## we compute the Profit or loss
    
    if data['form']['report_new'] :
        cr.execute('select id from account_account WHERE active and company_id = %s',(data['form']['company_id'],) )
        ids = map(lambda x: x[0], cr.fetchall())
        tmpid = False
        for account in pool.get('account.account').browse(cr, uid, ids,context={'fiscalyear': fy_id}):
            if account.user_type.close_method=='none' or account.type == 'view':
                continue
            if account.user_type.close_method.lower()=='balance':
                if abs(account.balance)>0.0001:
                    tmpid = pool.get('account.move.line').create(cr, uid, {
                        'debit': account.balance>0 and account.balance,
                        'credit': account.balance<0 and -account.balance,
                        'name': data['form']['report_name']+' '+account.name,
                        'date': period.date_start,
                        'journal_id': fy2.start_journal_period_id.id,
                        'period_id': period.id,
                        'account_id': account.id
                    }, {'journal_id':  fy2.start_journal_period_id.id, 'period_id':period.id})

            if account.user_type.close_method.lower()=='unreconciled':
                offset = 0
                limit = 100
                while True:
                    cr.execute('SELECT id, name, quantity, debit, credit, account_id, ref, ' \
                                'amount_currency, currency_id, blocked, partner_id, ' \
                                'date_maturity, date_created, origin_link ' \
                            'FROM account_move_line ' \
                            'WHERE account_id = %s ' \
                                'AND ' + query_line + ' ' \
                            'ORDER BY id ' \
                            'LIMIT %s OFFSET %s', (account.id, limit, offset))
                    result = cr.dictfetchall()
                    result = _filter_result_to_reconcile(cr, uid, result, fy, period,  pool)
                    if result is None:
                        break
                    for move in result:
                        move.update({
                            'name': data['form']['report_name']+' '+move['name'],
                            'date': period.date_start,
                            'journal_id': fy2.start_journal_period_id.id,
                            'period_id': period.id,
                        })
                        pool.get('account.move.line').create(cr, uid, move, {
                            'journal_id': fy2.start_journal_period_id.id,
                            'period_id': period.id,
                            })
                    offset += limit
                #we get the move id
            
            if account.user_type.close_method.lower()=='detail':
                offset = 0
                limit = 100
                while True:
                    cr.execute('SELECT name, quantity, debit, credit, account_id, ref, ' \
                                'amount_currency, currency_id, blocked, partner_id, ' \
                                'date_maturity, date_created ' \
                            'FROM account_move_line ' \
                            'WHERE account_id = %s ' \
                                'AND ' + query_line + ' ' \
                            'ORDER BY id ' \
                            'LIMIT %s OFFSET %s', (account.id,fy_id, limit, offset))
                    result = cr.dictfetchall()
                    if not result:
                        break
                    for move in result:
                        move.update({
                            'name': data['form']['report_name']+' '+move['name'],
                            'date': period.date_start,
                            'journal_id':  fy2.start_journal_period_id.id,
                            'period_id': period.id,
                        })
                        pool.get('account.move.line').create(cr, uid, move)
                    offset += limit
        m_id = pool.get('account.move.line').browse(cr, uid, tmpid).move_id.id
#        reconcile_new_lines(cr, uid, fy2, period, m_id, pool)
        # Redo Reconcile between closed year and opening year
        pool.get('account.move.line').reconcile_two_years(cr, uid, m_id, context=context)        
    _profLoss_to_bilan(cr, uid, data, fy, fy2, pool,m_id)    
    return {}

class wiz_journal_close(wizard.interface):
    states = {
        'init': {
            'actions': [_data_load],
            'result': {
                'type': 'form', 
                'arch':_transaction_form, 
                'fields':_transaction_fields, 
                'state':[('end','Cancel'),('close','Close Fiscal Year')]
                    }
        },
        
        'close': {
            'actions': [_data_save],
            'result': {'type': 'state', 'state':'end'}
        }
    }
wiz_journal_close('fiscalyear_close.fiscalyear.close')
